from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin
from osdu_client.validation import validate_data

from .models import CopyRecordReferencesModel, MultiRecordIds, MultiRecordRequest, RecordBulkUpdateParam, ReplayRequest


class StorageAPIError(OSDUAPIError):
    pass


class StorageClient(OSDUAPIClient):
    service_path = "/api/storage/v2/"

    def update_records(
        self,
        *,
        x_collaboration: str | None = None,
        skipdupes: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        params = {}
        if skipdupes is not None:
            params["skipdupes"] = skipdupes

        url = urljoin(self.base_url, self.service_path, "records")
        response = requests.put(url, headers=headers, params=params)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def patch_records(
        self,
        *,
        query: dict,
        ops: list[dict],
        x_collaboration: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        data = {
            "query": query,
            "ops": ops,
        }

        if self.validation:
            validate_data(data, RecordBulkUpdateParam, StorageAPIError)

        url = urljoin(self.base_url, self.service_path, "records")
        response = requests.patch(url, headers=headers, json=data)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def update_records_copy(
        self,
        *,
        target: str | None = None,
        records: list[dict] | None = None,
        x_collaboration: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        data = {}
        if target is not None:
            data["target"] = target
        if records is not None:
            data["records"] = records

        if self.validation:
            validate_data(data, CopyRecordReferencesModel, StorageAPIError)

        url = urljoin(self.base_url, self.service_path, "records/copy")
        response = requests.put(url, headers=headers, json=data)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def delete_record(
        self,
        *,
        x_collaboration: str | None = None,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        url = urljoin(self.base_url, self.service_path, "records/%s:delete" % id)
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def create_records_delete(
        self,
        *,
        x_collaboration: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        url = urljoin(self.base_url, self.service_path, "records/delete")
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def query_records_from_kind(
        self,
        *,
        x_collaboration: str | None = None,
        cursor: str | None = None,
        limit: str | None = None,
        kind: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        params = {
            "kind": kind,
        }
        if cursor is not None:
            params["cursor"] = cursor
        if limit is not None:
            params["limit"] = limit

        url = urljoin(self.base_url, self.service_path, "query/records")
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def query_records(
        self,
        *,
        records: list[str],
        attributes: list[str] | None = None,
        x_collaboration: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        data = {
            "records": records,
        }
        if attributes is not None:
            data["attributes"] = attributes

        if self.validation:
            validate_data(data, MultiRecordIds, StorageAPIError)

        url = urljoin(self.base_url, self.service_path, "query/records")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def query_records_batch(
        self,
        *,
        records: list[str],
        x_collaboration: str | None = None,
        frame_of_reference: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        headers.update(
            {
                "frame-of-reference": frame_of_reference,
            }
        )
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        data = {
            "records": records,
        }

        if self.validation:
            validate_data(data, MultiRecordRequest, StorageAPIError)

        url = urljoin(self.base_url, self.service_path, "query/records:batch")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def get_record(
        self,
        *,
        x_collaboration: str | None = None,
        id: str,
        attribute: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        params = {}
        if attribute is not None:
            params["attribute"] = attribute

        url = urljoin(self.base_url, self.service_path, "records/%s" % id)
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def purge_record(
        self,
        *,
        x_collaboration: str | None = None,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        url = urljoin(self.base_url, self.service_path, "records/%s" % id)
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def get_record_version(
        self,
        *,
        x_collaboration: str | None = None,
        id: str,
        version: str,
        attribute: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        params = {}
        if attribute is not None:
            params["attribute"] = attribute

        url = urljoin(self.base_url, self.service_path, "records/%s/%s" % (id, version))
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def get_record_versions(
        self,
        *,
        x_collaboration: str | None = None,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        url = urljoin(self.base_url, self.service_path, "records/versions/%s" % id)
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def get_liveness_check(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "liveness_check")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def get_info(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "info")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def purge_record_versions(
        self,
        *,
        x_collaboration: str | None = None,
        id: str,
        version_ids: str | None = None,
        limit: str | None = None,
        _form: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if x_collaboration is not None:
            headers["x-collaboration"] = x_collaboration

        params = {}
        if version_ids is not None:
            params["versionIds"] = version_ids
        if limit is not None:
            params["limit"] = limit
        if _form is not None:
            params["from"] = _form

        url = urljoin(self.base_url, self.service_path, "records/%s/versions" % id)
        response = requests.delete(url, headers=headers, params=params)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def get_replay_status(
        self,
        *,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "replay/status/%s" % id)
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def create_replay(
        self,
        *,
        operation: str,
        filter: Union[list[str]] | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "operation": operation,
        }
        if filter is not None:
            data["filter"] = filter

        if self.validation:
            validate_data(data, ReplayRequest, StorageAPIError)

        url = urljoin(self.base_url, self.service_path, "replay")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def get_whoami(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "whoami")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def update_whoami(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "whoami")
        response = requests.put(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def create_whoami(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "whoami")
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def delete_whoami(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "whoami")
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def options_whoami(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "whoami")
        response = requests.options(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def head_whoami(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "whoami")
        response = requests.head(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()

    def patch_whoami(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "whoami")
        response = requests.patch(url, headers=headers)
        if not response.ok:
            raise StorageAPIError(response.text, response.status_code)
        return response.json()



from .v2 import WellboreClient as WellboreClientV2  # NOQA
from .v3 import WellboreClient as WellboreClientV3  # NOQA

DEFAULT_VERSION = "v3"
VERSIONS = {
    "v2": WellboreClientV2,
    "v3": WellboreClientV3,
}



# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:17+00:00

from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field, RootModel

from .FileCollection.SEGY.field_1 import field_0 as field_0_1
from .FileCollection.Slb.OpenZGY.field_1 import field_0


class Status(BaseModel):
    status: str


class Info(BaseModel):
    group_id: Optional[str] = None
    artifact_id: Optional[str] = None
    build_time: Optional[str] = None
    connected_outer_services: Optional[List] = None


class DatasetListBody(BaseModel):
    type: Optional[str] = None
    gtags: Optional[List[str]] = Field(
        None,
        description="Array of global tags associated with the dataset metadata. Once assigned, they can be used to filter datasets.",
    )
    search: Optional[str] = None
    filter: Optional[Dict[str, Any]] = None
    limit: Optional[float] = Field(
        None, description="the maximum number of datasets in the response"
    )
    cursor: Optional[str] = Field(
        None,
        description="the cursor value required to retrieve the next page of result",
    )


class DatasetLsBody(BaseModel):
    sdpath: str = Field(
        ...,
        description="the seismic dms path uri: sd://tenant or sd://tenant/subproject or sd://tenant/subproject/path.",
    )
    wmode: Optional[str] = Field(
        None,
        description="the endpoint working mode: dirs, datasets, all (default if not specified).",
    )
    limit: Optional[float] = Field(
        None, description="the max response datasets number."
    )
    cursor: Optional[str] = Field(None, description="the next page cursor.")


class Seismicmeta(BaseModel):
    kind: Optional[str] = None
    data: Optional[Dict[str, Any]] = None


class Acls(BaseModel):
    admins: Optional[List[str]] = None
    viewers: Optional[List[str]] = None


class Dataset(BaseModel):
    name: str = Field(..., description="Name of the dataset.")
    tenant: str = Field(..., description="Name of the tenant.")
    subproject: str = Field(..., description="Name of the subproject.")
    path: str = Field(..., description="Path to the dataset inside a subproject.")
    created_by: str = Field(..., description="Creator of the dataset.")
    created_date: str = Field(..., description="Date when the dataset was created.")
    last_modified_date: str = Field(
        ..., description="Date when the dataset was last modified."
    )
    filemetadata: Optional[Dict[str, Any]] = Field(
        None, description="Number of objects and the size in bytes of the dataset."
    )
    metadata: Optional[Dict[str, Any]] = Field(
        None,
        description="Generic information stored as key,value pairs for the dataset.",
    )
    gcsurl: str = Field(..., description="Cloud storage object identifier.")
    readonly: Optional[bool] = Field(
        None, description="True if the dataset is readonly."
    )
    status: Optional[str] = Field(None, description="The dataset status (if set).")
    ltag: Optional[str] = Field(
        None, description="Legal tag associated with the dataset."
    )
    ctag: str = Field(
        ...,
        description="The coherency tag associated with the dataset. (It changes every time the dataset is updated.)",
    )
    seismicmeta_guid: Optional[str] = Field(
        None, description="Seismic storage record identifier."
    )
    sbit: Optional[str] = Field(None, description="The session lockID.")
    sbit_count: Optional[float] = Field(
        None, description="The number of sessions associated to the dataset."
    )
    seismicmeta: Optional[Seismicmeta] = Field(
        None,
        description="Seismic metadata associated with the dataset which is used to create a data ecosystem storage record.",
    )
    openzgy_v1: Optional[field_0.Field0] = None
    segy_v1: Optional[field_0_1.Field0] = None


class DatasetPatch(BaseModel):
    dataset_new_name: Optional[str] = Field(
        None, description="New name for the dataset."
    )
    metadata: Optional[Dict[str, Any]] = Field(
        None,
        description="Generic information about the dataset stored as key value pairs.",
    )
    filemetadata: Optional[Dict[str, Any]] = Field(
        None, description="Number of objects and the size in bytes of the dataset."
    )
    last_modified_date: Optional[str] = Field(
        None, description="Date when the dataset was last modified."
    )
    gtags: Optional[List[str]] = Field(
        [],
        description="Array of tags associated with the dataset. After patching these tags, they can be used to filter the datasets.",
    )
    ltag: Optional[str] = Field(
        None, description="Legal tag associated with the dataset."
    )
    readonly: Optional[bool] = Field(
        None, description="True if the dataset is read only."
    )
    status: Optional[str] = Field(
        None, description="The status of the dataset (if set)"
    )
    seismicmeta: Optional[Seismicmeta] = Field(
        None,
        description="Seismic metadata associated with the dataset which is used to create a data ecosystem storage record.",
    )
    openzgy_v1: Optional[field_0.Field0] = None
    segy_v1: Optional[field_0_1.Field0] = None
    acls: Optional[Acls] = Field(
        None, description="ACLs with admin groups and viewer groups for the dataset."
    )


class DatasetPermission(BaseModel):
    read: bool = Field(
        ..., description="True if the user has read permission on the dataset."
    )
    write: bool = Field(
        ..., description="True if the user has write permission on the dataset."
    )
    delete: bool = Field(
        ..., description="True if the user has delete permission on the dataset."
    )


class DatasetAndDirectories(BaseModel):
    datasets: List[str] = Field(
        ..., description="Array of datasets inside the subproject."
    )
    directories: List[str] = Field(
        ..., description="Array of directories inside the subproject."
    )


class DatasetCheckList(BaseModel):
    datasets: List[str] = Field(
        ..., description="Array of datasets inside the subproject."
    )


class DatasetBulkDeleteBody(BaseModel):
    filter: Optional[Dict[str, Any]] = Field(
        None,
        description="Optional structured query filter to restrict the datasets to be deleted.",
    )


class AccessToken(BaseModel):
    access_token: str = Field(
        ...,
        description="connection string credentials or standard access token (CSP dependent)",
    )
    token_type: str = Field(..., description="token type (Bearer, SasUrl, ....).")
    expires_in: float = Field(
        ...,
        description="expiration time (in minutes) of the connection string credentials",
    )


class ImpersonationToken(BaseModel):
    impersonation_token: str = Field(..., description="The Impersonation token.")
    token_type: str = Field(..., description="Token type.")
    expires_in: float = Field(..., description="Token expiration time.")
    context: str = Field(..., description="the Impersonation token context.")


class Resource(BaseModel):
    resource: str
    readonly: Optional[bool] = None


class ImpersonationTokenRequest(BaseModel):
    resources: List[Resource]
    metadata: Optional[Dict[str, Any]] = Field(
        None,
        description="A general field to set extra meta-information to the impersonation token signature. These are saved along with the token signatures.",
    )


class ImpTokenRequest(BaseModel):
    token: str = Field(
        ..., description="Impersonation token that was previously issued."
    )
    resources: List[Resource]
    refresh_url: str = Field(
        ...,
        alias="refresh-url",
        description="A web service endpoint that the seismic store service will invoke to check if the impersonation token can be refreshed.",
    )


class ImpTokenPatchRequest(BaseModel):
    token: str = Field(
        ..., description="Impersonation token that was previously issued."
    )
    refresh_url: str = Field(
        ...,
        alias="refresh-url",
        description="New endpoint that the seismic store service will invoke to check if the impersonation token can be refreshed.",
    )


class RefreshTokenRequest(BaseModel):
    token: str = Field(..., description="Impersonation token")


class AccessPolicy(Enum):
    uniform = "uniform"
    dataset = "dataset"


class SubProjectCreateBody(BaseModel):
    admin: Optional[str] = Field(
        None,
        description="An optional user to set as admin. The user will be added in the first admins acl group",
    )
    storage_class: Optional[str] = Field(
        None, description="Storage class for the bucket (Google Required Only)"
    )
    storage_location: Optional[str] = Field(
        None, description="Storage location for the bucket (Google Required Only)"
    )
    access_policy: Optional[AccessPolicy] = Field(
        "uniform",
        description='The datasets access level mode: "uniform" (uniform data access to all subprojects datasets) or "datasets" (acl can be applied at dataset level)',
    )
    acls: Optional[Acls] = Field(
        None,
        description="The entitlement groups to enable subproject access as admin or viewer. If not specified, default entitlement data groups will be created.",
    )


class SubProjectPatchBody(BaseModel):
    access_policy: Optional[str] = Field(
        "dataset", description="Access Policy for the subproject."
    )
    acls: Optional[Acls] = Field(
        None,
        description="ACLs with admin groups and viewer groups. Existing acls will be replaced with the provided acls.",
    )


class SubProject(BaseModel):
    name: str = Field(..., description="Name of the subproject.")
    tenant: str = Field(..., description="Name of the tenant.")
    storage_class: str = Field(..., description="Storage class for the bucket.")
    storage_location: str = Field(..., description="Storage location for the bucket.")
    ltag: str = Field(..., description="Legal tag for the subproject.")
    access_policy: AccessPolicy = Field(
        ..., description="Access policy for the subproject."
    )
    acls: Optional[Acls] = Field(
        None, description="ACLs with admin groups and viewer groups for the subproject."
    )


class TenantCreateBody(BaseModel):
    gcpid: str = Field(
        ...,
        description="Google cloud project id associated with the tenant. For other providers, name of the data partition.",
    )
    esd: str = Field(
        ...,
        description="Entitlements group Sub Domain (ESD). For instance, if the entitlements group is users.datalake.viewers@{datapartition}.{domain}.com, the esd value is {datapartition}.{domain}.com. It must start with the name of the data partition.",
    )
    default_acls: str = Field(
        ...,
        description="Entitlements authorization group to manage tenant administrators.",
    )


class Tenant(BaseModel):
    name: str = Field(..., description="Name of the tenant.")
    esd: str = Field(
        ...,
        description="Entitlements group Sub Domain (ESD). For instance, if the entitlements group is users.datalake.viewers@{datapartition}.{domain}.com, the esd value is {datapartition}.{domain}.com. It has to start with the name of the data partition.",
    )
    gcpid: str = Field(
        ...,
        description="Google cloud project id associated with the tenant. For other providers, name of the data partition.",
    )
    default_acls: str = Field(
        ...,
        description="Entitlements authorization group to manage tenant administrators.",
    )


class Group(Enum):
    viewer = "viewer"
    admin = "admin"


class UserAddRequest(BaseModel):
    email: str = Field(
        ...,
        description="Email address of the user. Note that this field is used as a principal identifier for entitlements-svc and can be configured using the env variable USER_ID_CLAIM_FOR_ENTITLEMENTS_SVC in the Seismic DMS runtime.",
    )
    path: str = Field(
        ..., description="Seismic store path in the format sd://tenant/subproject."
    )
    group: Group = Field(..., description="Role to be assigned to the user.")


class UserRemoveRequest(BaseModel):
    email: str = Field(
        ...,
        description="Email address of the user. Note that this field is used as a principal identifier for entitlements-svc and can be configured using the env variable USER_ID_CLAIM_FOR_ENTITLEMENTS_SVC in the Seismic DMS runtime.",
    )
    path: str = Field(
        ..., description="Seismic store path in the format sd://tenant/subproject."
    )


class UserRoles(BaseModel):
    roles: List[List[str]] = Field(
        ...,
        description="Array of arrays with each array containing subproject and role of the user for the subproject.",
        example=[["/spx01", "admin"], ["/spx02", "viewer"]],
    )


class LsDatasets(RootModel[List[str]]):
    root: List[str] = Field(
        ...,
        description="Array of directories and datasets inside a subproject.",
        example=["folderA/", "folderB/", "dataset01", "dataset02", "dataset03"],
    )


class LsPaginatedDatasets(BaseModel):
    datasets: List[str] = Field(
        ..., description="Array of datasets inside the subproject."
    )
    nextPageCursor: str = Field(..., description="Next cursor for pagination.")


class PaginatedDatasets(BaseModel):
    datasets: Optional[List[Dataset]] = None
    nextPageCursor: Optional[str] = Field(
        None, example="next-page-token-3rt431v41-4515351"
    )


class DatasetSize(BaseModel):
    computed_size: float = Field(..., description="The size of the dataset in bytes..")
    computed_size_date: str = Field(
        ..., description="The date of when the dateset size was computed."
    )


class ComputedSize(BaseModel):
    size_bytes: float = Field(..., description="The size of the datasets in bytes.")
    dataset_count: float = Field(
        ...,
        description="The number of datasets in the path (including nested folders).",
    )


class Operation(BaseModel):
    operation_id: str = Field(..., description="Operation id of the operation job.")


class OperationStatus(BaseModel):
    operation_id: str = Field(..., description="Operation id of the operation job.")
    created_at: Optional[str] = Field(
        None, description="Time when the operation was initiated."
    )
    created_by: Optional[str] = Field(
        None, description="User who initiated the operation job."
    )
    last_updated_at: Optional[str] = Field(
        None, description="Time when the operation status was last updated."
    )
    status: str = Field(..., description="Status of the operation job.")
    dataset_cnt: Optional[float] = Field(
        None, description="Total number of datasets to process."
    )
    completed_cnt: Optional[float] = Field(
        None, description="Number of datasets correctly processed."
    )
    failed_cnt: Optional[float] = Field(
        None, description="Number of datasets which processing failed."
    )


class DatasetRegisterBody(BaseModel):
    type: Optional[str] = None
    gtags: Optional[List[str]] = Field(
        None,
        description="Array of global tags associated with the dataset metadata. Once assigned, they can be used to filter datasets.",
    )
    seismicmeta: Optional[Seismicmeta] = Field(
        None,
        description="Seismic metadata to be stored as dataecosystem storage record.",
    )
    openzgy_v1: Optional[field_0.Field0] = None
    segy_v1: Optional[field_0_1.Field0] = None
    acls: Optional[Acls] = Field(
        None, description="ACLs with admin groups and viewer groups for the dataset."
    )



from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.rafs.common import RAFSCommonClient
from osdu_client.utils import urljoin


class RAFSAPIError(OSDUAPIError):
    pass


class RAFSClient(RAFSCommonClient):
    service_path = ""

    def get_sar_record(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysesreport/%s" % record_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def soft_delete_sar_record(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysesreport/%s" % record_id,
        )
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_sar_record_versions(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysesreport/%s/versions" % record_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_sar_record_specific_version(
        self,
        *,
        version: str,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysesreport/%s/versions/%s"
            % (version, record_id),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def create_or_update_sar_records(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "api/rafs-ddms/v2/samplesanalysesreport"
        )
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_sar_source_data(
        self,
        *,
        record_id: str,
        version: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if version is not None:
            params["version"] = version

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysesreport/%s/source" % record_id,
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_sa_types(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysis/analysistypes",
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_sa_record(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysis/%s" % record_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def soft_delete_sa_record(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysis/%s" % record_id,
        )
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_sa_record_versions(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysis/%s/versions" % record_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_sa_record_version(
        self,
        *,
        version: str,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysis/%s/versions/%s" % (version, record_id),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def create_or_update_sa_records(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "api/rafs-ddms/v2/samplesanalysis"
        )
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_sa_content_schema(
        self,
        *,
        analysistype: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysis/%s/data/schema" % analysistype,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_sa_data(
        self,
        *,
        record_id: str,
        analysis_type: str,
        dataset_id: str,
        columns_filter: str | None = None,
        rows_filter: str | None = None,
        columns_aggregation: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if columns_filter is not None:
            params["columns_filter"] = columns_filter
        if rows_filter is not None:
            params["rows_filter"] = rows_filter
        if columns_aggregation is not None:
            params["columns_aggregation"] = columns_aggregation

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysis/%s/data/%s/%s"
            % (record_id, analysis_type, dataset_id),
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def upload_sa_data(
        self,
        *,
        record_id: str,
        analysis_type: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysis/%s/data/%s" % (record_id, analysis_type),
        )
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def search_sa_data(
        self,
        *,
        analysis_type: str,
        columns_filter: str | None = None,
        rows_filter: str | None = None,
        columns_aggregation: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if columns_filter is not None:
            params["columns_filter"] = columns_filter
        if rows_filter is not None:
            params["rows_filter"] = rows_filter
        if columns_aggregation is not None:
            params["columns_aggregation"] = columns_aggregation

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysis/%s/search/data" % analysis_type,
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def query_sa(
        self,
        *,
        analysis_type: str,
        columns_filter: str | None = None,
        rows_filter: str | None = None,
        columns_aggregation: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if columns_filter is not None:
            params["columns_filter"] = columns_filter
        if rows_filter is not None:
            params["rows_filter"] = rows_filter
        if columns_aggregation is not None:
            params["columns_aggregation"] = columns_aggregation

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/samplesanalysis/%s/search" % analysis_type,
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_masterdata_record(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/masterdata/%s" % record_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def soft_delete_masterdata_record(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/masterdata/%s" % record_id,
        )
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_masterdata_record_versions(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/masterdata/%s/versions" % record_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_masterdata_record_version(
        self,
        *,
        version: str,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/masterdata/%s/versions/%s" % (version, record_id),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def create_or_update_masterdata_records(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "api/rafs-ddms/v2/masterdata")
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_pvt_record(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "api/rafs-ddms/v2/pvtmodel/%s" % record_id
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def soft_delete_pvt_record(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "api/rafs-ddms/v2/pvtmodel/%s" % record_id
        )
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_pvt_record_versions(
        self,
        *,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/pvtmodel/%s/versions" % record_id,
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_pvt_record_version(
        self,
        *,
        version: str,
        record_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/pvtmodel/%s/versions/%s" % (version, record_id),
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def create_or_update_pvt_records(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "api/rafs-ddms/v2/pvtmodel")
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def get_pvt_data(
        self,
        *,
        record_id: str,
        analysis_type: str,
        dataset_id: str,
        columns_filter: str | None = None,
        rows_filter: str | None = None,
        columns_aggregation: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if columns_filter is not None:
            params["columns_filter"] = columns_filter
        if rows_filter is not None:
            params["rows_filter"] = rows_filter
        if columns_aggregation is not None:
            params["columns_aggregation"] = columns_aggregation

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/pvtmodel/%s/data/%s/%s"
            % (record_id, analysis_type, dataset_id),
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()

    def upload_pvt_data(
        self,
        *,
        record_id: str,
        analysis_type: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "api/rafs-ddms/v2/pvtmodel/%s/data/%s" % (record_id, analysis_type),
        )
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise RAFSAPIError(response.text, response.status_code)
        return response.json()



# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:20+00:00

from __future__ import annotations

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field


class ResponseEntity(BaseModel):
    body: Optional[Dict[str, Any]] = None
    statusCode: Optional[str] = None
    statusCodeValue: Optional[int] = None


class ConnectedOuterService(BaseModel):
    name: Optional[str] = Field(None, description="Connected outer service name.")
    version: Optional[str] = Field(None, description="Connected outer service version.")


class VersionInfo(BaseModel):
    groupId: Optional[str] = Field(None, description="Maven artifact group ID.")
    actifactId: Optional[str] = Field(None, description="Maven artifact ID.")
    version: Optional[str] = Field(None, description="Maven artifact version")
    buildTime: Optional[str] = Field(None, description="Maven artifact build time")
    branch: Optional[str] = Field(None, description="Current git branch")
    commitId: Optional[str] = Field(None, description="Latest commit hash")
    commitMessage: Optional[str] = Field(None, description="Latest commit message")
    connectedOuterServices: Optional[List[ConnectedOuterService]] = Field(
        None, description="Connected outer services information"
    )



from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin
from osdu_client.validation import validate_data

from .models import CursorQueryRequest, QueryRequest


class SearchAPIError(OSDUAPIError):
    pass


class SearchClient(OSDUAPIClient):
    service_path = "/api/search/v2/"

    def query_with_cursor(
        self,
        *,
        kind: dict,
        limit: int | None = None,
        query: str | None = None,
        highlighted_fields: list[str] | None = None,
        returned_fields: list[str] | None = None,
        sort: dict | None = None,
        query_as_owner: bool | None = None,
        track_total_count: bool | None = None,
        spatial_filter: dict | None = None,
        cursor: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "kind": kind,
        }
        if limit is not None:
            data["limit"] = limit
        if query is not None:
            data["query"] = query
        if highlighted_fields is not None:
            data["highlightedFields"] = highlighted_fields
        if returned_fields is not None:
            data["returnedFields"] = returned_fields
        if sort is not None:
            data["sort"] = sort
        if query_as_owner is not None:
            data["queryAsOwner"] = query_as_owner
        if track_total_count is not None:
            data["trackTotalCount"] = track_total_count
        if spatial_filter is not None:
            data["spatialFilter"] = spatial_filter
        if cursor is not None:
            data["cursor"] = cursor

        if self.validation:
            validate_data(data, CursorQueryRequest, SearchAPIError)

        url = urljoin(self.base_url, self.service_path, "query_with_cursor")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise SearchAPIError(response.text, response.status_code)
        return response.json()

    def query(
        self,
        *,
        kind: dict,
        limit: int | None = None,
        query: str | None = None,
        highlighted_fields: list[str] | None = None,
        returned_fields: list[str] | None = None,
        sort: dict | None = None,
        query_as_owner: bool | None = None,
        track_total_count: bool | None = None,
        spatial_filter: dict | None = None,
        aggregate_by: str | None = None,
        offset: int | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "kind": kind,
        }
        if limit is not None:
            data["limit"] = limit
        if query is not None:
            data["query"] = query
        if highlighted_fields is not None:
            data["highlightedFields"] = highlighted_fields
        if returned_fields is not None:
            data["returnedFields"] = returned_fields
        if sort is not None:
            data["sort"] = sort
        if query_as_owner is not None:
            data["queryAsOwner"] = query_as_owner
        if track_total_count is not None:
            data["trackTotalCount"] = track_total_count
        if spatial_filter is not None:
            data["spatialFilter"] = spatial_filter
        if aggregate_by is not None:
            data["aggregateBy"] = aggregate_by
        if offset is not None:
            data["offset"] = offset

        if self.validation:
            validate_data(data, QueryRequest, SearchAPIError)

        url = urljoin(self.base_url, self.service_path, "query")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise SearchAPIError(response.text, response.status_code)
        return response.json()

    def get_readiness_check(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "readiness_check")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SearchAPIError(response.text, response.status_code)
        return response.json()

    def get_liveness_check(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "liveness_check")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SearchAPIError(response.text, response.status_code)
        return response.json()

    def get_info(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "info")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise SearchAPIError(response.text, response.status_code)
        return response.json()



from __future__ import annotations

from pydantic import BaseModel, ValidationError

from osdu_client.exceptions import OSDUAPIError
from osdu_client.utils import convert_to_snake_case


def parse_loc(loc: list[str]) -> str:
    _loc = []
    for i, el in enumerate(loc):
        if i == 0:
            _loc.append(convert_to_snake_case(el))
        else:
            _loc.append(str(el))
    return ".".join(_loc)


def validate_data(data: dict, schema: BaseModel, exception: OSDUAPIError = OSDUAPIError):
    try:
        schema(**data)
    except ValidationError as e:
        messages = []
        for error in e.errors(include_url=False):
            _type = error["type"]
            loc = parse_loc(error["loc"])
            msg = error["msg"]
            _input = error["input"]
            messages.append(f"{msg} {_type} in {loc}. Input: {_input}")
        raise exception(messages)



from __future__ import annotations

from abc import ABCMeta, abstractmethod


class AuthBackendInterface(metaclass=ABCMeta):

    def get_headers(self) -> dict:
        headers = {}
        headers.update(self.authorization_header)
        headers.update(
            {
                "tenant": self.default_tenant,
                "data-partition-id": self.default_data_partition_id
            }
        )
        return headers

    @property
    @abstractmethod
    def default_tenant(self) -> str:
        pass

    @property
    @abstractmethod
    def default_data_partition_id(self) -> str:
        pass

    @property
    @abstractmethod
    def authorization_header(self) -> dict:
        pass

    @property
    @abstractmethod
    def base_url(self) -> str:
        pass

    @abstractmethod
    def get_sd_connection_params(self, log_level: int = None) -> dict:
        pass



# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:18+00:00

from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, confloat, conint


class Point(BaseModel):
    latitude: Optional[confloat(ge=-90.0, le=90.0)] = None
    longitude: Optional[float] = None


class Polygon(BaseModel):
    points: Optional[List[Point]] = None


class OrderEnum(Enum):
    ASC = "ASC"
    DESC = "DESC"


class SortQuery(BaseModel):
    field: Optional[List[str]] = None
    order: Optional[List[OrderEnum]] = None
    filter: Optional[List[str]] = None


class CursorQueryResponse(BaseModel):
    cursor: Optional[str] = None
    results: Optional[List[Dict[str, Dict[str, Any]]]] = None
    totalCount: Optional[int] = None


class AppError(BaseModel):
    code: Optional[int] = None
    reason: Optional[str] = None
    message: Optional[str] = None


class AggregationResponse(BaseModel):
    key: Optional[str] = None
    count: Optional[int] = None


class QueryResponse(BaseModel):
    results: Optional[List[Dict[str, Dict[str, Any]]]] = None
    aggregations: Optional[List[AggregationResponse]] = None
    totalCount: Optional[int] = None


class ConnectedOuterService(BaseModel):
    name: Optional[str] = None
    version: Optional[str] = None


class VersionInfo(BaseModel):
    groupId: Optional[str] = None
    artifactId: Optional[str] = None
    version: Optional[str] = None
    buildTime: Optional[str] = None
    branch: Optional[str] = None
    commitId: Optional[str] = None
    commitMessage: Optional[str] = None
    connectedOuterServices: Optional[List[ConnectedOuterService]] = None


class ByBoundingBox(BaseModel):
    topLeft: Point
    bottomRight: Point


class ByDistance(BaseModel):
    distance: Optional[confloat(le=9.223372036854776e18)] = None
    point: Point


class ByGeoPolygon(BaseModel):
    points: List[Point]


class ByIntersection(BaseModel):
    polygons: List[Polygon]


class ByWithinPolygon(BaseModel):
    points: List[Point]


class SpatialFilter(BaseModel):
    field: str
    byBoundingBox: Optional[ByBoundingBox] = None
    byDistance: Optional[ByDistance] = None
    byGeoPolygon: Optional[ByGeoPolygon] = None
    byIntersection: Optional[ByIntersection] = None
    byWithinPolygon: Optional[ByWithinPolygon] = None


class QueryRequest(BaseModel):
    kind: Dict[str, Any]
    limit: Optional[conint(ge=0)] = None
    query: Optional[str] = None
    highlightedFields: Optional[List[str]] = None
    returnedFields: Optional[List[str]] = None
    sort: Optional[SortQuery] = None
    queryAsOwner: Optional[bool] = None
    trackTotalCount: Optional[bool] = None
    spatialFilter: Optional[SpatialFilter] = None
    aggregateBy: Optional[str] = None
    offset: Optional[conint(ge=0)] = None


class CursorQueryRequest(BaseModel):
    kind: Dict[str, Any]
    limit: Optional[conint(ge=0)] = None
    query: Optional[str] = None
    highlightedFields: Optional[List[str]] = None
    returnedFields: Optional[List[str]] = None
    sort: Optional[SortQuery] = None
    queryAsOwner: Optional[bool] = None
    trackTotalCount: Optional[bool] = None
    spatialFilter: Optional[SpatialFilter] = None
    cursor: Optional[str] = None



from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin
from osdu_client.validation import validate_data

from .models import StatusDto


class PWSAPIError(OSDUAPIError):
    pass


class PWSClient(OSDUAPIClient):
    service_path = "/api/pws/v1/"

    def get_projects(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "projects")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def create_project(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "projects")
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def change_projects_status(
        self,
        *,
        status: str | None = None,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {}
        if status is not None:
            data["status"] = status

        if self.validation:
            validate_data(data, StatusDto, PWSAPIError)

        url = urljoin(self.base_url, self.service_path, "projects/%s/status" % id)
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def get_project_resources(
        self,
        *,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "projects/%s/resources" % id)
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def assign_projects_resources(
        self,
        *,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "projects/%s/resources" % id)
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def delete_projects_resources(
        self,
        *,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "projects/%s/resources" % id)
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def get_projects_lifecycleevent(
        self,
        *,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "projects/%s/lifecycleevent" % id
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def assign_projects_lifecycleevent(
        self,
        *,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "projects/%s/lifecycleevent" % id
        )
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def delete_projects_lifecycleevent(
        self,
        *,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "projects/%s/lifecycleevent" % id
        )
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def get_project(
        self,
        *,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "projects/%s" % id)
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def get_projects_wip_resources(
        self,
        *,
        id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url, self.service_path, "projects/%s/wip-resources" % id
        )
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def get_info(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "info")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def get_readiness_check(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "_ah/readiness_check")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()

    def get_liveness_check(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "_ah/liveness_check")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PWSAPIError(response.text, response.status_code)
        return response.json()



from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin
from osdu_client.validation import validate_data

from .models import PartitionInfo


class PartitionAPIError(OSDUAPIError):
    pass


class PartitionClient(OSDUAPIClient):
    service_path = "/api/partition/v1"

    def get_partition(
        self,
        *,
        partition_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "partitions/%s" % partition_id)
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PartitionAPIError(response.text, response.status_code)
        return response.json()

    def create_partitions(
        self,
        *,
        properties: dict,
        partition_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "properties": properties,
        }

        if self.validation:
            validate_data(data, PartitionInfo, PartitionAPIError)

        url = urljoin(self.base_url, self.service_path, "partitions/%s" % partition_id)
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise PartitionAPIError(response.text, response.status_code)
        return response.json()

    def delete_partition(
        self,
        *,
        partition_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "partitions/%s" % partition_id)
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise PartitionAPIError(response.text, response.status_code)
        return response.json()

    def update_partitions(
        self,
        *,
        properties: dict,
        partition_id: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "properties": properties,
        }

        if self.validation:
            validate_data(data, PartitionInfo, PartitionAPIError)

        url = urljoin(self.base_url, self.service_path, "partitions/%s" % partition_id)
        response = requests.patch(url, headers=headers, json=data)
        if not response.ok:
            raise PartitionAPIError(response.text, response.status_code)
        return response.json()

    def list_partitions(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "partitions")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PartitionAPIError(response.text, response.status_code)
        return response.json()

    def get_liveness_check(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "liveness_check")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PartitionAPIError(response.text, response.status_code)
        return response.json()

    def get_info(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "info")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise PartitionAPIError(response.text, response.status_code)
        return response.json()



from __future__ import annotations

import requests

from osdu_client.exceptions import OSDUAPIError
from osdu_client.services.base import OSDUAPIClient
from osdu_client.utils import urljoin


class EntitlementsAPIError(OSDUAPIError):
    pass


class EntitlementsClient(OSDUAPIClient):
    service_path = "/api/entitlements/v2"

    def get_liveness_check(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "_ah/liveness_check")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def get_readiness_check(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "_ah/readiness_check")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def get_groups(
        self,
        *,
        on_behalf_of: str | None = None,
        role_required: str | None = False,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant
        if on_behalf_of is not None:
            headers["on-behalf-of"] = on_behalf_of

        params = {}
        if role_required is not None:
            params["roleRequired"] = role_required

        url = urljoin(self.base_url, self.service_path, "groups")
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def create_group(
        self,
        *,
        group_info_dto: dict,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "groupInfoDto": group_info_dto,
        }

        url = urljoin(self.base_url, self.service_path, "groups")
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def delete_group(
        self,
        *,
        group_email: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if group_email is not None:
            params["groupEmail"] = group_email

        url = urljoin(self.base_url, self.service_path, "groups/%s")
        response = requests.delete(url, headers=headers, params=params)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def update_groups(
        self,
        *,
        update_group_request: list[dict],
        group_email: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "updateGroupRequest": update_group_request,
        }

        url = urljoin(self.base_url, self.service_path, "groups/%s" % group_email)
        response = requests.patch(url, headers=headers, json=data)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def get_groups_members(
        self,
        *,
        group_email: str,
        role: str | None = None,
        include_type: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if role is not None:
            params["role"] = role
        if include_type is not None:
            params["includeType"] = include_type

        url = urljoin(
            self.base_url, self.service_path, "groups/%s/members" % group_email
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def add_member(
        self,
        *,
        add_member_dto: dict,
        group_email: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        data = {
            "addMemberDto": add_member_dto,
        }

        url = urljoin(
            self.base_url, self.service_path, "groups/%s/members" % group_email
        )
        response = requests.post(url, headers=headers, json=data)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def delete_member_from_group(
        self,
        *,
        group_email: str,
        member_email: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(
            self.base_url,
            self.service_path,
            "groups/%s/members/%s" % (group_email, member_email),
        )
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def delete_member(
        self,
        *,
        member_email: str,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "members/%s" % member_email)
        response = requests.delete(url, headers=headers)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def get_members_groups(
        self,
        *,
        member_email: str,
        type: str,
        appid: str | None = None,
        role_required: str | None = False,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "type": type,
        }
        if appid is not None:
            params["appid"] = appid
        if role_required is not None:
            params["roleRequired"] = role_required

        url = urljoin(
            self.base_url, self.service_path, "members/%s/groups" % member_email
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def initiate_tenant(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "tenant-provisioning")
        response = requests.post(url, headers=headers)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def get_info(
        self, data_partition_id: str | None = None, tenant: str | None = None
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        url = urljoin(self.base_url, self.service_path, "info")
        response = requests.get(url, headers=headers)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def get_count_group_members(
        self,
        *,
        group_email: str,
        role: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {}
        if role is not None:
            params["role"] = role

        url = urljoin(
            self.base_url, self.service_path, "groups/%s/membersCount" % group_email
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()

    def list_partition_groups(
        self,
        *,
        type: str,
        cursor: str | None = None,
        limit: str | None = None,
        data_partition_id: str | None = None,
        tenant: str | None = None,
    ) -> dict:
        headers = self.auth.get_headers()
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        if tenant:
            headers["tenant"] = tenant

        params = {
            "type": type,
        }
        if cursor is not None:
            params["cursor"] = cursor
        if limit is not None:
            params["limit"] = limit

        url = urljoin(
            self.base_url, self.service_path, "api/entitlements/v2/groups/all"
        )
        response = requests.get(url, headers=headers, params=params)
        if not response.ok:
            raise EntitlementsAPIError(response.text, response.status_code)
        return response.json()



# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:24+00:00

from __future__ import annotations

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, Field


class Acl(BaseModel):
    viewers: List[str] = Field(..., title="Viewers")
    owners: List[str] = Field(..., title="Owners")


class InfoResponse(BaseModel):
    name: Optional[str] = Field(None, title="Name")
    app_version: Optional[str] = Field(None, title="App Version")
    build_time: Optional[str] = Field(None, title="Build Time")
    branch: Optional[str] = Field(None, title="Branch")
    commit_id: Optional[str] = Field(None, title="Commit Id")
    commit_message: Optional[str] = Field(None, title="Commit Message")
    release_version: Optional[str] = Field(None, title="Release Version")


class Legal(BaseModel):
    legaltags: List[str] = Field(..., title="Legaltags")
    otherRelevantDataCountries: List[str] = Field(
        ..., title="Otherrelevantdatacountries"
    )
    status: Optional[str] = Field(None, title="Status")


class OsduStorageRecord(BaseModel):
    id: Optional[str] = Field(None, title="Id")
    kind: str = Field(..., title="Kind")
    acl: Acl
    legal: Legal
    data: Dict[str, Any] = Field(..., title="Data")
    meta: Optional[List] = Field(None, title="Meta")
    ancestry: Optional[Dict[str, Any]] = Field(None, title="Ancestry")
    tags: Optional[Dict[str, Any]] = Field(None, title="Tags")
    version: Optional[int] = Field(None, title="Version")
    createUser: Optional[str] = Field(None, title="Createuser")
    createTime: Optional[str] = Field(None, title="Createtime")
    modifyUser: Optional[str] = Field(None, title="Modifyuser")
    modifyTime: Optional[str] = Field(None, title="Modifytime")


class StorageUpsertResponse(BaseModel):
    record_count: int = Field(..., title="Record Count")
    record_id_versions: List[str] = Field(..., title="Record Id Versions")
    skipped_record_count: int = Field(..., title="Skipped Record Count")


class ValidationError(BaseModel):
    loc: List[Union[str, int]]
    msg: str
    type: str


class HTTPValidationError(BaseModel):
    errors: Optional[List[ValidationError]] = None
